from setuptools import setup

setup(name='Gaus_dotgb_Bin',
      version='0.1',
      description='Gaussian and Binomial distributions',
      packages=['Gaus_dotgb_Bin'],
      author = 'dotgb',
      author_email = 'myemail@something.com',
      zip_safe=False)

